@@ -147,22 +147,31 @@ static bool cleanup_directory( const string& directory )
         if( strcmp( f->d_name, "." ) == 0 || strcmp( f->d_name, ".." ) == 0 )
             continue;
         string fullpath = directory + '/' + f->d_name;
-        if( unlink( fullpath.c_str()) != 0 ) {
-            if( !cleanup_directory( fullpath ) || rmdir( fullpath.c_str()) != 0 ) {
+        struct stat st;
+        if ( lstat( fullpath.c_str(), &st )) {
+            perror( "stat" );
+            return false;
+        }
+        if ( S_ISDIR( st.st_mode )) {
+            if( !cleanup_directory( fullpath ) || rmdir( fullpath.c_str()) != 0 )
+                return false;
+        } else {
+            if( unlink( fullpath.c_str()) != 0 )
                 return false;
-            }
         }
     }
     closedir( dir );
     return true;
 }
 
-bool cleanup_cache( const string &basedir )
+bool cleanup_cache( const string &basedir, uid_t user_uid, gid_t user_gid )
 {
     flush_debug();
 
-    if ( access( basedir.c_str(), R_OK ) == 0 && !cleanup_directory( basedir ))
+    if ( access( basedir.c_str(), R_OK ) == 0 && !cleanup_directory( basedir )) {
+        log_error() << "failed to clean up envs dir" << endl;
         return false;
+    }
 
     if ( mkdir( basedir.c_str(), 0755 ) && errno != EEXIST ) {
         if ( errno == EPERM )
@@ -171,6 +180,11 @@ bool cleanup_cache( const string &basedir )
             log_perror( "mkdir in cleanup_cache() failed" );
         return false;
     }
+    if ( chown( basedir.c_str(), user_uid, user_gid ) ||
+         chmod( basedir.c_str(), 0775 ) ) {
+        log_perror( "chown/chmod in cleanup_cache() failed" );
+        return false;
+    }
 
     return true;
 }
@@ -531,6 +545,8 @@ void chdir_to_environment( MsgChannel *client, const string &dirname, uid_t user
         log_perror("chroot() failed" );
         _exit(144);
     }
+    (void) user_uid;
+    (void) user_gid;
 #else
     if ( getuid() == 0 ) {
         // without the chdir, the chroot will escape the
